'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.pwd = exports.cd = exports.mkdir = exports.cat = exports.ls = exports.clear = exports.help = undefined;

var _util = require('./util');

var Util = _interopRequireWildcard(_util);

var _const = require('./const');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var helpCommands = ['clear', 'ls', 'cat', 'mkdir', 'cd', 'pwd'];

var help = exports.help = {
    exec: function exec(_ref) {
        var history = _ref.history;
        var structure = _ref.structure;
        var cwd = _ref.cwd;

        return { structure: structure, cwd: cwd,
            history: history.concat.apply(history, [{ value: 'React-bash:' }, { value: 'These shell commands are defined internally.  Type \'help\' to see this list.' }].concat(_toConsumableArray(helpCommands.map(function (value) {
                return { value: value };
            }))))
        };
    }
};

var clear = exports.clear = {
    exec: function exec(_ref2) {
        var structure = _ref2.structure;
        var cwd = _ref2.cwd;

        return { structure: structure, cwd: cwd, history: [] };
    }
};

var ls = exports.ls = {
    aliases: {
        '-l': '--long',
        '-la': ['--long', '--all'],
        '-al': ['--long', '--all'],
        '-a': '--all'
    },
    exec: function exec(state, args) {
        var history = state.history;
        var structure = state.structure;
        var cwd = state.cwd;

        var path = args[0] || '';
        var fullPath = Util.extractPath(path, cwd);

        var _Util$getDirectoryByP = Util.getDirectoryByPath(structure, fullPath);

        var err = _Util$getDirectoryByP.err;
        var dir = _Util$getDirectoryByP.dir;


        if (err) {
            return Util.reportError(state, err, path);
        } else {
            var content = Object.keys(dir);
            if (!args.all) {
                content = content.filter(function (name) {
                    return name[0] !== '.';
                });
            }
            if (args.long) {
                return { structure: structure, cwd: cwd,
                    history: history.concat(content.map(function (value) {
                        return { value: value };
                    }))
                };
            } else {
                return { structure: structure, cwd: cwd,
                    history: history.concat({ value: content.join(' ') })
                };
            }
        }
    }
};

var cat = exports.cat = {
    exec: function exec(state, args) {
        var history = state.history;
        var structure = state.structure;
        var cwd = state.cwd;

        var path = args[0];
        var relativePath = path.split('/');
        var fileName = relativePath.pop();
        var fullPath = Util.extractPath(relativePath.join('/'), cwd);

        var _Util$getDirectoryByP2 = Util.getDirectoryByPath(structure, fullPath);

        var err = _Util$getDirectoryByP2.err;
        var dir = _Util$getDirectoryByP2.dir;

        if (err) {
            return Util.reportError(state, err, path);
        } else if (!dir[fileName]) {
            return Util.reportError(state, _const.Errors.NO_SUCH_FILE, path);
        } else if (!dir[fileName].hasOwnProperty('content')) {
            return Util.reportError(state, _const.Errors.IS_A_DIRECTORY, path);
        } else {
            return { cwd: cwd, structure: structure,
                history: history.concat({
                    value: dir[fileName].content
                })
            };
        }
    }
};

var mkdir = exports.mkdir = {
    exec: function exec(state, args) {
        var history = state.history;
        var structure = state.structure;
        var cwd = state.cwd;

        var path = args[0];
        var relativePath = path.split('/');
        var newDirectory = relativePath.pop();
        var fullPath = Util.extractPath(relativePath.join('/'), cwd);
        var deepCopy = JSON.parse(JSON.stringify(structure));

        var _Util$getDirectoryByP3 = Util.getDirectoryByPath(deepCopy, fullPath);

        var dir = _Util$getDirectoryByP3.dir;


        if (dir[newDirectory]) {
            return Util.reportError(state, _const.Errors.FILE_EXISTS, path);
        } else {
            dir[newDirectory] = {};
            return { cwd: cwd, history: history, structure: deepCopy };
        }
    }
};

var cd = exports.cd = {
    exec: function exec(state, args) {
        var history = state.history;
        var structure = state.structure;
        var cwd = state.cwd;

        var path = args[0];
        if (!path || path === '/') {
            return { structure: structure, history: history, cwd: '' };
        }

        var fullPath = Util.extractPath(path, cwd);

        var _Util$getDirectoryByP4 = Util.getDirectoryByPath(structure, fullPath);

        var err = _Util$getDirectoryByP4.err;


        if (err) {
            return Util.reportError(state, err, path);
        } else {
            return { history: history, structure: structure, cwd: fullPath };
        }
    }
};

var pwd = exports.pwd = {
    exec: function exec(state) {
        var history = state.history;
        var structure = state.structure;
        var cwd = state.cwd;

        var directory = '/' + cwd;

        return {
            cwd: cwd, structure: structure,
            history: history.concat({ value: directory })
        };
    }
};