'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.trim = trim;
exports.reportError = reportError;
exports.extractPath = extractPath;
exports.getDirectoryByPath = getDirectoryByPath;

var _const = require('./const');

function trim(str, char) {
    if (str[0] === char) {
        str = str.substr(1);
    }
    if (str[str.length - 1] === char) {
        str = str.substr(0, str.length - 1);
    }
    return str;
}

function reportError(state, error, command) {
    return Object.assign({}, state, {
        history: state.history.concat({
            value: error.replace('$1', command)
        })
    });
}

/*
 * This is a utility method for chaining the relativePath
 * onto the rootPath. It includes backwards movement and
 * normalization.
 */
function extractPath(relativePath, rootPath) {
    // Short circuit for relative path
    if (relativePath === '') return rootPath;

    // Strip trailing slash
    relativePath = trim(relativePath, '/');

    // Create raw path
    var path = '' + (rootPath ? rootPath + '/' : '') + relativePath;

    // Strip ../ references
    while (path.match(_const.BACK_REGEX)) {
        path = path.replace(_const.BACK_REGEX, '');
    }
    return trim(path, '/');
}

/*
 * This is a utility method for traversing the <structure>
 * down the '/' separated <relativePath>
 */
function getDirectoryByPath(structure, relativePath) {
    var path = relativePath.split('/');

    // Short circuit for empty root path
    if (!path[0]) return { dir: structure };

    var dir = structure;
    var i = 0;
    while (i < path.length) {
        var key = path[i];
        var child = dir[key];
        if (child && (typeof child === 'undefined' ? 'undefined' : _typeof(child)) === 'object') {
            if (child.hasOwnProperty('content')) {
                return { err: _const.Errors.NOT_A_DIRECTORY.replace('$1', relativePath) };
            } else {
                dir = child;
            }
        } else {
            return { err: _const.Errors.NO_SUCH_FILE.replace('$1', relativePath) };
        }
        i++;
    }
    return { dir: dir };
}